﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.SurrogateProblem {
  [StorableClass]
  [Item("AngularDistance", "The Angluar Distance as defined as a normalized distance measure dependent on the angle between two vectors.\nIt is designed for vectors with all positive coordinates")]
  public class AngularDistance : DistanceBase<IReadOnlyList<double>> {

    #region HLConstructors
    [StorableConstructor]
    protected AngularDistance(bool deserializing) : base(deserializing) { }
    protected AngularDistance(AngularDistance original, Cloner cloner)
      : base(original, cloner) { }
    public AngularDistance() { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new AngularDistance(this, cloner);
    }
    #endregion

    #region statics
    public static double GetDistance(IReadOnlyList<double> point1, IReadOnlyList<double> point2) {
      if (point1.Count != point2.Count) throw new ArgumentException("Angluar distance not defined on vectors of different length");
      var innerProduct = point1.Zip(point2, (x, y) => x * y).Sum();
      var res = Math.Acos(innerProduct / (point1.Norm() * point2.Norm())) / Math.PI;
      return double.IsNaN(res) ? 0 : res;
    }
    public static double GetDistance(IReadOnlyList<double> a, IReadOnlyList<double> b, double threshold) {
      return GetDistance(a, b);     //no shortcut evaluation for Angular Distance
    }
    #endregion
    public override double Get(IReadOnlyList<double> a, IReadOnlyList<double> b) {
      return GetDistance(a, b);
    }
    public override double Get(IReadOnlyList<double> a, IReadOnlyList<double> b, double threshold) {
      return GetDistance(a, b, threshold);
    }
  }
}
